@@ -153,19 +153,97 @@ impl<'a> RenderScope<'a> {
         render_object(tmpl, &ctx, out)
     }
 
+    #[allow(
+        clippy::cognitive_complexity,
+        reason = "Keeps string omission logic in one place for clarity."
+    )]
     fn serialize_string(&mut self, id: usize) -> String {
         let kept = self.count_kept_children(id);
-        let omitted = self.omitted_for(id, kept).unwrap_or(0);
+        // Number of graphemes to render from the string prefix, honoring any
+        // free-prefix allowance enabled in lines-only mode.
+        let render_prefix_graphemes =
+            match self.config.string_free_prefix_graphemes {
+                Some(n) => kept.max(n),
+                None => kept,
+            };
+        let omitted =
+            self.omitted_for(id, render_prefix_graphemes).unwrap_or(0);
         let full: &str = match &self.order.nodes[id] {
             RankedNode::SplittableLeaf { value, .. } => value.as_str(),
             _ => unreachable!(
                 "serialize_string called for non-string node: id={id}"
             ),
         };
-        if omitted == 0 {
+        if matches!(
+            self.config.template,
+            crate::serialization::types::OutputTemplate::Text
+        ) {
+            if omitted == 0 {
+                full.to_string()
+            } else {
+                let prefix = crate::utils::text::take_n_graphemes(
+                    full,
+                    render_prefix_graphemes,
+                );
+                format!("{prefix}…")
+            }
+        } else if omitted == 0 {
             crate::utils::json::json_string(full)
         } else {
-            let prefix = crate::utils::text::take_n_graphemes(full, kept);
+            let prefix = crate::utils::text::take_n_graphemes(
+                full,
+                render_prefix_graphemes,
+            );
+            let truncated = format!("{prefix}…");
+            crate::utils::json::json_string(&truncated)
+        }
+    }
+
+    #[allow(
+        clippy::cognitive_complexity,
+        reason = "Keeps string omission logic in one place for clarity."
+    )]
+    fn serialize_string_with_template(
+        &mut self,
+        id: usize,
+        template: crate::serialization::types::OutputTemplate,
+    ) -> String {
+        let kept = self.count_kept_children(id);
+        // Number of graphemes to render from the string prefix, honoring any
+        // free-prefix allowance enabled in lines-only mode.
+        let render_prefix_graphemes =
+            match self.config.string_free_prefix_graphemes {
+                Some(n) => kept.max(n),
+                None => kept,
+            };
+        let omitted =
+            self.omitted_for(id, render_prefix_graphemes).unwrap_or(0);
+        let full: &str = match &self.order.nodes[id] {
+            RankedNode::SplittableLeaf { value, .. } => value.as_str(),
+            _ => unreachable!(
+                "serialize_string called for non-string node: id={id}"
+            ),
+        };
+        if matches!(
+            template,
+            crate::serialization::types::OutputTemplate::Text
+        ) {
+            if omitted == 0 {
+                full.to_string()
+            } else {
+                let prefix = crate::utils::text::take_n_graphemes(
+                    full,
+                    render_prefix_graphemes,
+                );
+                format!("{prefix}…")
+            }
+        } else if omitted == 0 {
+            crate::utils::json::json_string(full)
+        } else {
+            let prefix = crate::utils::text::take_n_graphemes(
+                full,
+                render_prefix_graphemes,
+            );
             let truncated = format!("{prefix}…");
             crate::utils::json::json_string(&truncated)
         }
@@ -194,7 +272,15 @@ impl<'a> RenderScope<'a> {
             }
             RankedNode::SplittableLeaf { .. } => {
                 let s = self.serialize_string(id);
-                out.push_string_literal(&s);
+                if matches!(
+                    self.config.template,
+                    crate::serialization::types::OutputTemplate::Text
+                ) {
+                    // For text template, push raw string without quotes or color.
+                    out.push_str(&s);
+                } else {
+                    out.push_string_literal(&s);
+                }
             }
             RankedNode::AtomicLeaf { .. } => {
                 let s = self.serialize_atomic(id);
@@ -459,7 +545,9 @@ impl<'a> RenderScope<'a> {
                 );
                 s
             }
-            RankedNode::SplittableLeaf { .. } => self.serialize_string(id),
+            RankedNode::SplittableLeaf { .. } => {
+                self.serialize_string_with_template(id, template)
+            }
             RankedNode::AtomicLeaf { .. } => self.serialize_atomic(id),
             RankedNode::LeafPart { .. } => {
                 unreachable!("string part not rendered")
@@ -546,7 +634,7 @@ mod tests {
 
     #[test]
     fn arena_render_empty_array() {
-        let arena = crate::json_ingest::build_json_tree_arena(
+        let arena = crate::ingest::formats::json::build_json_tree_arena(
             "[]",
             &crate::PriorityConfig::new(usize::MAX, usize::MAX),
         )
@@ -571,6 +659,7 @@ mod tests {
                 color_mode: crate::ColorMode::Auto,
                 color_enabled: false,
                 style: crate::serialization::types::Style::Strict,
+                string_free_prefix_graphemes: None,
             },
         );
         assert_snapshot!("arena_render_empty", out);
@@ -580,7 +669,7 @@ mod tests {
     fn newline_detection_crlf_array_child() {
         // Ensure we exercise the render_has_newline branch that checks
         // arbitrary newline sequences (e.g., "\r\n") via s.contains(nl).
-        let arena = crate::json_ingest::build_json_tree_arena(
+        let arena = crate::ingest::formats::json::build_json_tree_arena(
             "[{\"a\":1,\"b\":2}]",
             &crate::PriorityConfig::new(usize::MAX, usize::MAX),
         )
@@ -606,6 +695,7 @@ mod tests {
                 color_mode: crate::ColorMode::Auto,
                 color_enabled: false,
                 style: crate::serialization::types::Style::Strict,
+                string_free_prefix_graphemes: None,
             },
         );
         // Sanity: output should contain CRLF newlines and render the object child across lines.
@@ -618,7 +708,7 @@ mod tests {
 
     #[test]
     fn arena_render_single_string_array() {
-        let arena = crate::json_ingest::build_json_tree_arena(
+        let arena = crate::ingest::formats::json::build_json_tree_arena(
             "[\"ab\"]",
             &crate::PriorityConfig::new(usize::MAX, usize::MAX),
         )
@@ -643,6 +733,7 @@ mod tests {
                 color_mode: crate::ColorMode::Auto,
                 color_enabled: false,
                 style: crate::serialization::types::Style::Strict,
+                string_free_prefix_graphemes: None,
             },
         );
         assert_snapshot!("arena_render_single", out);
@@ -657,10 +748,12 @@ mod tests {
             prefer_tail_arrays: false,
             array_bias: crate::ArrayBias::HeadMidTail,
             array_sampler: crate::ArraySamplerStrategy::Default,
+            line_budget_only: false,
         };
-        let arena =
-            crate::json_ingest::build_json_tree_arena("[1,2,3]", &cfg_prio)
-                .unwrap();
+        let arena = crate::ingest::formats::json::build_json_tree_arena(
+            "[1,2,3]", &cfg_prio,
+        )
+        .unwrap();
         let build = build_order(&arena, &cfg_prio).unwrap();
         let mut marks = vec![0u32; build.total_nodes];
 
@@ -679,6 +772,7 @@ mod tests {
                 color_mode: crate::ColorMode::Off,
                 color_enabled: false,
                 style: crate::serialization::types::Style::Default,
+                string_free_prefix_graphemes: None,
             },
         );
         assert_snapshot!("array_omitted_pseudo_head", out_head);
@@ -698,6 +792,7 @@ mod tests {
                 color_mode: crate::ColorMode::Off,
                 color_enabled: false,
                 style: crate::serialization::types::Style::Default,
+                string_free_prefix_graphemes: None,
             },
         );
         assert_snapshot!("array_omitted_pseudo_tail", out_tail);
@@ -711,10 +806,12 @@ mod tests {
             prefer_tail_arrays: false,
             array_bias: crate::ArrayBias::HeadMidTail,
             array_sampler: crate::ArraySamplerStrategy::Default,
+            line_budget_only: false,
         };
-        let arena =
-            crate::json_ingest::build_json_tree_arena("[1,2,3]", &cfg_prio)
-                .unwrap();
+        let arena = crate::ingest::formats::json::build_json_tree_arena(
+            "[1,2,3]", &cfg_prio,
+        )
+        .unwrap();
         let build = build_order(&arena, &cfg_prio).unwrap();
         let mut marks = vec![0u32; build.total_nodes];
 
@@ -732,6 +829,7 @@ mod tests {
                 color_mode: crate::ColorMode::Off,
                 color_enabled: false,
                 style: crate::serialization::types::Style::Detailed,
+                string_free_prefix_graphemes: None,
             },
         );
         assert_snapshot!("array_omitted_js_head", out_head);
@@ -750,6 +848,7 @@ mod tests {
                 color_mode: crate::ColorMode::Off,
                 color_enabled: false,
                 style: crate::serialization::types::Style::Detailed,
+                string_free_prefix_graphemes: None,
             },
         );
         assert_snapshot!("array_omitted_js_tail", out_tail);
@@ -763,10 +862,12 @@ mod tests {
             prefer_tail_arrays: false,
             array_bias: crate::ArrayBias::HeadMidTail,
             array_sampler: crate::ArraySamplerStrategy::Default,
+            line_budget_only: false,
         };
-        let arena =
-            crate::json_ingest::build_json_tree_arena("[1,2,3]", &cfg_prio)
-                .unwrap();
+        let arena = crate::ingest::formats::json::build_json_tree_arena(
+            "[1,2,3]", &cfg_prio,
+        )
+        .unwrap();
         let build = build_order(&arena, &cfg_prio).unwrap();
         let mut marks = vec![0u32; build.total_nodes];
 
@@ -784,6 +885,7 @@ mod tests {
                 color_mode: crate::ColorMode::Off,
                 color_enabled: false,
                 style: crate::serialization::types::Style::Detailed,
+                string_free_prefix_graphemes: None,
             },
         );
         assert_yaml_valid(&out_head);
@@ -803,6 +905,7 @@ mod tests {
                 color_mode: crate::ColorMode::Off,
                 color_enabled: false,
                 style: crate::serialization::types::Style::Detailed,
+                string_free_prefix_graphemes: None,
             },
         );
         assert_yaml_valid(&out_tail);
@@ -811,7 +914,7 @@ mod tests {
 
     #[test]
     fn arena_render_empty_array_yaml() {
-        let arena = crate::json_ingest::build_json_tree_arena(
+        let arena = crate::ingest::formats::json::build_json_tree_arena(
             "[]",
             &crate::PriorityConfig::new(usize::MAX, usize::MAX),
         )
@@ -836,6 +939,7 @@ mod tests {
                 color_mode: crate::ColorMode::Auto,
                 color_enabled: false,
                 style: crate::serialization::types::Style::Default,
+                string_free_prefix_graphemes: None,
             },
         );
         assert_yaml_valid(&out);
@@ -844,7 +948,7 @@ mod tests {
 
     #[test]
     fn arena_render_single_string_array_yaml() {
-        let arena = crate::json_ingest::build_json_tree_arena(
+        let arena = crate::ingest::formats::json::build_json_tree_arena(
             "[\"ab\"]",
             &crate::PriorityConfig::new(usize::MAX, usize::MAX),
         )
@@ -869,6 +973,7 @@ mod tests {
                 color_mode: crate::ColorMode::Auto,
                 color_enabled: false,
                 style: crate::serialization::types::Style::Default,
+                string_free_prefix_graphemes: None,
             },
         );
         assert_yaml_valid(&out);
@@ -877,7 +982,7 @@ mod tests {
 
     #[test]
     fn inline_open_array_in_object_yaml() {
-        let arena = crate::json_ingest::build_json_tree_arena(
+        let arena = crate::ingest::formats::json::build_json_tree_arena(
             "{\"a\":[1,2,3]}",
             &crate::PriorityConfig::new(usize::MAX, 2),
         )
@@ -900,6 +1005,7 @@ mod tests {
                 color_mode: crate::ColorMode::Off,
                 color_enabled: false,
                 style: crate::serialization::types::Style::Detailed,
+                string_free_prefix_graphemes: None,
             },
         );
         assert_yaml_valid(&out);
@@ -935,7 +1041,7 @@ mod tests {
     fn yaml_key_and_scalar_quoting() {
         // Keys and values that exercise YAML quoting heuristics.
         let json = "{\n            \"true\": 1,\n            \"010\": \"010\",\n            \"-dash\": \"ok\",\n            \"normal\": \"simple\",\n            \"a:b\": \"a:b\",\n            \" spaced \": \" spaced \",\n            \"reserved\": \"yes\",\n            \"multiline\": \"line1\\nline2\"\n        }";
-        let arena = crate::json_ingest::build_json_tree_arena(
+        let arena = crate::ingest::formats::json::build_json_tree_arena(
             json,
             &crate::PriorityConfig::new(usize::MAX, usize::MAX),
         )
@@ -960,6 +1066,7 @@ mod tests {
                 color_mode: crate::ColorMode::Off,
                 color_enabled: false,
                 style: crate::serialization::types::Style::Default,
+                string_free_prefix_graphemes: None,
             },
         );
         assert_yaml_valid(&out);
@@ -1004,7 +1111,7 @@ mod tests {
     fn string_parts_never_rendered_but_affect_truncation() {
         // Build a long string: the string node itself is SplittableLeaf; the
         // builder also creates LeafPart children used only for priority.
-        let arena = crate::json_ingest::build_json_tree_arena(
+        let arena = crate::ingest::formats::json::build_json_tree_arena(
             "\"abcdefghij\"",
             &crate::PriorityConfig::new(usize::MAX, usize::MAX),
         )
@@ -1030,6 +1137,7 @@ mod tests {
                 color_mode: crate::ColorMode::Off,
                 color_enabled: false,
                 style: crate::serialization::types::Style::Strict,
+                string_free_prefix_graphemes: None,
             },
         );
         // Expect the first 5 characters plus an ellipsis, as a valid JSON string literal.
@@ -1038,7 +1146,7 @@ mod tests {
 
     #[test]
     fn yaml_array_of_objects_indentation() {
-        let arena = crate::json_ingest::build_json_tree_arena(
+        let arena = crate::ingest::formats::json::build_json_tree_arena(
             "[{\"a\":1,\"b\":2},{\"x\":3}]",
             &crate::PriorityConfig::new(usize::MAX, usize::MAX),
         )
@@ -1063,6 +1171,7 @@ mod tests {
                 color_mode: crate::ColorMode::Off,
                 color_enabled: false,
                 style: crate::serialization::types::Style::Default,
+                string_free_prefix_graphemes: None,
             },
         );
         assert_yaml_valid(&out);
@@ -1080,7 +1189,7 @@ mod tests {
     #[test]
     fn omitted_for_atomic_returns_none() {
         // Single atomic value as input (number), root is AtomicLeaf.
-        let arena = crate::json_ingest::build_json_tree_arena(
+        let arena = crate::ingest::formats::json::build_json_tree_arena(
             "1",
             &crate::PriorityConfig::new(usize::MAX, usize::MAX),
         )
@@ -1103,6 +1212,7 @@ mod tests {
             color_mode: crate::ColorMode::Off,
             color_enabled: false,
             style: crate::serialization::types::Style::Strict,
+            string_free_prefix_graphemes: None,
         };
         let scope = RenderScope {
             order: &build,
@@ -1117,7 +1227,7 @@ mod tests {
 
     #[test]
     fn inline_open_array_in_object_json() {
-        let arena = crate::json_ingest::build_json_tree_arena(
+        let arena = crate::ingest::formats::json::build_json_tree_arena(
             "{\"a\":[1,2,3]}",
             &crate::PriorityConfig::new(usize::MAX, 2),
         )
@@ -1140,6 +1250,7 @@ mod tests {
                 color_mode: crate::ColorMode::Off,
                 color_enabled: false,
                 style: crate::serialization::types::Style::Strict,
+                string_free_prefix_graphemes: None,
             },
         );
         assert_snapshot!("inline_open_array_in_object_json", out);
@@ -1148,7 +1259,7 @@ mod tests {
     #[test]
     fn arena_render_object_partial_js() {
         // Object with three properties; render top_k small so only one child is kept.
-        let arena = crate::json_ingest::build_json_tree_arena(
+        let arena = crate::ingest::formats::json::build_json_tree_arena(
             "{\"a\":1,\"b\":2,\"c\":3}",
             &crate::PriorityConfig::new(usize::MAX, usize::MAX),
         )
@@ -1174,6 +1285,7 @@ mod tests {
                 color_mode: crate::ColorMode::Auto,
                 color_enabled: false,
                 style: crate::serialization::types::Style::Detailed,
+                string_free_prefix_graphemes: None,
             },
         );
         // Should be a valid JS object with one property and an omitted summary.
